﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;

namespace WindowsFormsApplication1
{
    [DataContract]
    public struct TWLine
    {
        [DataMember]
        public double[] w;
    }

    [Serializable]
    public class TAutoEncoderData
    {
        [DataMember]
        public TWLine[] AutoEncoderLine;

        [DataMember]
        public double[] mean;

        [DataMember]
        public double[] deviation;
    }

    public class Data
    {
        public Collection<double[]> values = new Collection<double[]>();
        public int dataLength = 0;
        private int features;
        private int reduced;
        public double[] mean; 
        public double[] deviation;
        public double[,] v;

        public Data(int iFeatures, int iReduced)
        {
            features = iFeatures;
            reduced = iReduced;
        }

        public void readData(string FileName)
        {
            foreach (var str in File.ReadLines(FileName, Encoding.Default))
            {
                int i;
                string[] strLine = str.Split(',');
                double[] line = new double[strLine.Length];
                for (i = 0; i < strLine.Length; i++)
                    line[i] = Convert.ToDouble(strLine[i]);
                dataLength = line.Length;
                values.Add(line);
            }
        }


        public void writeTrainedData(string FileName)
        {
            int i, j;
            TAutoEncoderData parameter = new TAutoEncoderData();
            parameter.AutoEncoderLine = new TWLine[features];
            //byte[] startBytes = Encoding.ASCII.GetBytes("{\n \"AutoencoderData\": [");
            //byte[] eol = Encoding.ASCII.GetBytes(",\n");

            MemoryStream ms = new MemoryStream();
            //ms.Write(startBytes, 0, startBytes.Length);
            DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(TAutoEncoderData));
            parameter.mean = mean;
            for (i = 0; i < features; i++)
            {
                parameter.AutoEncoderLine[i].w = new double[reduced];
                for (j = 0; j < reduced; j++)
                {
                    parameter.AutoEncoderLine[i].w[j] = v[i, j];
                }
            }
            ser.WriteObject(ms, parameter);
            //if (i < features - 1)
            //    ms.Write(eol, 0, eol.Length);
            //byte[] endBytes = Encoding.ASCII.GetBytes("]}");
            //ms.Write(endBytes, 0, endBytes.Length);

            using (FileStream fs = new FileStream(FileName, FileMode.Create))
            {
                ms.Position = 0;
                fs.Write(ms.ToArray(), 0, ms.ToArray().Length);
                ms.Close();
                fs.Close();
            }
        }


        public void readAutoEncoderData(string FileName)
        {
            int i, j;
            using (StreamReader r = new StreamReader(FileName))
            {
                string response = r.ReadToEnd();
                TAutoEncoderData parameter = new TAutoEncoderData();
                parameter.AutoEncoderLine = new TWLine[features];
                for (i = 0; i < features; i++)
                {
                    parameter.AutoEncoderLine[i].w = new double[reduced];
                }
                parameter.mean = new double[features];
                parameter.deviation = new double[features];
                MemoryStream m = new MemoryStream(Encoding.UTF8.GetBytes(response));
                DataContractJsonSerializer ser = new DataContractJsonSerializer(parameter.GetType());
                parameter = ser.ReadObject(m) as TAutoEncoderData;
                mean = parameter.mean;
                deviation = parameter.deviation;
                v = new double[features, reduced];
                for (i = 0; i < features; i++)
                {
                    for (j = 0; j < reduced; j++)
                     {
                         v[i,j] = parameter.AutoEncoderLine[i].w[j];
                     }
                }
            }
        }
    }
}

