﻿using System;
using System.Linq;
using System.Windows.Forms;
using System.Collections.Generic;

/*                       Backpropagation Test                                    */
/*                                                                               */
/*                       www.mosismath.com                                       */

namespace WindowsFormsApplication1
{
   
    public partial class Mainwin : Form
    {

        int samples = 29;
        int dataLength = 0;
        const int features = 4;
        const int reduced = 2;
        int layers = 0;
       
        Data data = new Data(features, reduced);
        List<TLayer> net = new List<TLayer>();

        public Mainwin()
        {
            int i,j;
            DataGridViewCell cell;
            TLayer actLayer;

            InitializeComponent();
            data.readData(Application.StartupPath + "\\Data_iris_test.csv");
             
            actLayer = new TLayer(reduced, 2);
            net.Add(actLayer);
            actLayer = new TLayer(2, 2);
            net.Add(actLayer);

            layers = net.Count;
            data.readAutoEncoderData(Application.StartupPath + "\\autoencoderdata.json");

           
            actLayer = net.ElementAt(0); // initialisation of the neural net
            actLayer.w[0, 0] = -2.2058122400432358;
            actLayer.w[0, 1] = -3.0565455726172877;
            actLayer.w[1, 0] = 0.070283188101906521;
            actLayer.w[1, 1] = -0.69358551406553848;
           
            actLayer.offs[0] = -3.1315784770352431;
            actLayer.offs[1] = -9.0712405887413983;

            actLayer = net.ElementAt(1);
            actLayer.w[0, 0] = 100.57326422962251;
            actLayer.w[0, 1] = 26.802034332071138;
            actLayer.w[1, 0] = 132.21374281053969;
            actLayer.w[1, 1] = 52.097602809356189;

            actLayer.offs[0] = -10.111959295186813;
            actLayer.offs[1] = -14.212757859444654;

            samples = data.values.Count;
            dataLength = data.dataLength;
            dgDataView.RowCount = samples;
            dgDataView.ColumnCount = 10;// features + 3;

            dgDataView.Columns[0].HeaderText = "x1";
            dgDataView.Columns[1].HeaderText = "x2";
            dgDataView.Columns[2].HeaderText = "x3";
            dgDataView.Columns[3].HeaderText = "x4";
            dgDataView.Columns[5].HeaderText = "Res";
            dgDataView.Columns[6].HeaderText = "[%]";
            for (i = 0; i < samples; i++)
            {
                dgDataView.Rows[i].HeaderCell.Value = (i + 1).ToString();
                for (j = 0; j < features; j++)
                {
                    cell = dgDataView[j, i];
                    cell.Value = Math.Round(data.values.ElementAt(i)[j], 4).ToString();
                }
            }
        }

        public double[] MultMatrix(double[] m1, double[,] m2, int size, int sizeReduced)
        {
            int i, k;
            double[] al = new double[sizeReduced];
            for (i = 0; i < sizeReduced; i++)
            {
                al[i] = 0.0;
                for (k = 0; k < size; k++)
                {
                    al[i] = al[i] + (m1[k] * m2[k, i]);
                }
            }
            return al;
        }

        public double[] MultTransMatrix(double[] m1, double[,] m2, int size, int sizeReduced)
        {
            int i, k;
            double[] al = new double[sizeReduced];
            for (i = 0; i < sizeReduced; i++)
            {
                al[i] = 0.0;
                for (k = 0; k < size; k++)
                {
                    al[i] = al[i] + (m1[k] * m2[i, k]);
                }
            }
            return al;
        }

        private double[] ForwardProp(double[] x)
        {
            int i, j, k;
            TLayer actLayer = net.ElementAt(0);
            double[] actX = new double[x.Length];
            for (i = 0; i < x.Length; i++)
              actX[i] = x[i];

            for (i = 0; i < layers; i++)
            {
                for (j = 0; j < actLayer.featuresOut; j++)
                {
                    actLayer.x[j] = actLayer.offs[j];
                    for (k = 0; k < actLayer.featuresIn; k++)
                    {
                        actLayer.x[j] = actLayer.x[j] + actX[k] * actLayer.w[k, j];
                    }
                    actLayer.o[j] = actLayer.Act(actLayer.x[j]);
                }

                for (j = 0; j < actLayer.x.Length; j++)
                    actX[j] = actLayer.o[j];
              
                if(i < layers - 1)
                    actLayer = net.ElementAt(i + 1);
            }
            return actLayer.o;
        }

      
        private void bt_Calc_Click(object sender, EventArgs e)
        {
            int i, k;
            double[] res;
            double[] tempFeatures = new double[features];
            double[] tempX = new double[reduced];
            DataGridViewCell cell;
            TLayer actLayer = net.ElementAt(0);
           
            // print data
            for (i = 0; i < samples; i++)
            {
                tempFeatures = data.values.ElementAt(i);
               
                for (k = 0; k < features; k++)
                {
                    tempFeatures[k] = (tempFeatures[k] - data.mean[k])/data.deviation[k];
                }
                tempX = MultMatrix(tempFeatures, data.v, 4, 2);

                double[] tempVal = MultTransMatrix(tempX, data.v, 2, 4);

                res = ForwardProp(tempX);
                cell = dgDataView[5, i];
                if ((res[0] < 0.5) && (res[1] < 0.5))
                {
                    cell.Value = "Iris_setosa";
                }
                else
                {
                    if (res[1] < 0.5)
                    {
                        cell.Value = "Iris-versicolor";
                    }
                    else
                    {
                        cell.Value = "Iris_virginica";
                    }
                }
                double tempRes = 0;
                if (res[0] < 0.5)
                    tempRes = tempRes + (1 - res[0]) * (1 - res[0]);
                else
                    tempRes = tempRes + res[0] * res[0];
                if (res[1] < 0.5)
                    tempRes = tempRes + (1 - res[1]) * (1 - res[1]);
                else
                    tempRes = tempRes + res[1] * res[1];
                cell = dgDataView[6, i];
                cell.Value = Math.Round(Math.Sqrt(tempRes/2) * 100, 4).ToString();
            }
        }
    }
}
